# create a simulator object
set ns [new Simulator]



# open the nam trace file
set nam_trace_fd [open Queue_A.nam w]
$ns namtrace-all $nam_trace_fd
set trace_fd [open Queue_A.tr w]
set drop_trace_fd [open Queue_A_drop.tr w]

# open the measurement output files
set throughput_trace_fd(1) [open Queue_A_throughput1.tr w]
set throughput_trace_fd(2) [open Queue_A_throughput2.tr w]
set throughput_trace_fd(3) [open Queue_A_throughput3.tr w]
set throughput_trace_fd(4) [open Queue_A_throughput4.tr w]
set throughput_trace_fd(5) [open Queue_A_throughput5.tr w]
set throughput_trace_fd(6) [open Queue_A_throughput6.tr w]
set throughput_trace_fd(7) [open Queue_A_throughput7.tr w]
set throughput_trace_fd(8) [open Queue_A_throughput8.tr w]
set throughput_trace_fd(9) [open Queue_A_throughput9.tr w]
set throughput_trace_fd(0)  [open Queue_A_throughput0.tr w]



set packetSize 1000

# define different colors for nam data flows
$ns color 0 Green
$ns color 1 Blue
$ns color 2 Red
$ns color 3 Yellow
$ns color 4 Seagreen
$ns color 5 Purple
$ns color 6 Orange
$ns color 7 Pink
$ns color 8 Brown

# define a 'finish' procedure
proc finish {} {
        global ns nam_trace_fd trace_fd
        global trace_fd drop_trace_fd
   	global throughput_trace_fd
   
   
   
	# close the nam trace file
        $ns flush-trace

        close $nam_trace_fd
   	close $drop_trace_fd

 # close the measurement files
	 close $trace_fd
	 close $throughput_trace_fd(1)
	 close $throughput_trace_fd(2)
	 close $throughput_trace_fd(3)
	 close $throughput_trace_fd(4)
	 close $throughput_trace_fd(5)
	 close $throughput_trace_fd(6)
	 close $throughput_trace_fd(7)
	 close $throughput_trace_fd(8)
	 close $throughput_trace_fd(9)
	 close $throughput_trace_fd(0)
   
	# execute nam on the trace file
        # exec nam tcp_c.nam &
        exit 0
}

proc record_stat {} {
        global flow_monitor tcp packetSize flowstats
        global trace_fd throughput_trace_fd drop_trace_fd
        

	# get an instance of the simulator
	set ns [Simulator instance]

	# set the time after which the procedure should be called again
        set time 0.01

	# get the current time
        set now [$ns now]

        puts $drop_trace_fd "$now \t [expr [$flow_monitor set bdrops_]/$packetSize]"

	for {set i 0} {$i < 10} {incr i} {             
		# how many bytes have been received by the traffic sinks?
        	#set packet_arrival [$flowstats($i) set parrivals_]
        	#set packet_departure [$flowstats($i) set pdepartures_]
        	#set packet_drop [$flowstats($i) set pdrops_]
        	set flow_classifier [$flow_monitor classifier]
	
		set flow_fd [$flow_classifier lookup auto 0 0 $i]
		if { $flow_fd != "" } {
		
			# calculate the throughput and write it to the files
			puts $throughput_trace_fd($i) "$now \t [expr [$flow_fd set bdepartures_]*8/$time]"
	
			
			$flow_fd set barrivals_ 0
			$flow_fd set bdepartures_ 0
			$flow_fd set bdrops_ 0		
	
		    }
		# reset the bytes_ values on the traffic sinks
     	}
        $flow_monitor set barrivals_ 0
        $flow_monitor set bdepartures_ 0
        $flow_monitor set bdrops_ 0

        # re-schedule the procedure
        $ns at [expr $now+$time] "record_stat"
}


# create 22 nodes
for {set i 0} {$i < 22} {incr i} {
  set n($i) [$ns node]
}


# create links between the nodes
for {set i 0} {$i < 10} {incr i} {
    $ns duplex-link $n($i) $n(10) 1Mb 4ms DropTail
    set num [expr $i + 12]
    $ns duplex-link $n($num) $n(11) 1Mb 5ms DropTail
}

$ns duplex-link $n(10) $n(11) 5Mb 2ms Queue_A
$ns queue-limit $n(10) $n(11) 100


# monitor the queue for the link between node 10 and node 11
$ns duplex-link-op $n(10) $n(11) queuePos 0.5


# TCP traffic source
# create a TCP agent and attach it to nodes

for {set i 0} {$i < 10} {incr i} {
    set tcp($i) [new Agent/TCP/Reno]
    $ns attach-agent $n($i) $tcp($i)
    $tcp($i) set fid_ $i
    $tcp($i) set class_ $i
    
    # set packet length, please don't remove below line!!!
    $tcp($i) set window_ 9999
    # window_ * (packetsize_ + 40) / RTT
    #$tcp($i) set window_ 30
    
    set sink($i) [new Agent/TCPSink]
    set num [expr $i + 12]
    $ns attach-agent $n($num) $sink($i)
    # connect both agents
    $ns connect $tcp($i) $sink($i)

    # create an FTP source "application";
    set ftp($i) [new Application/FTP]
    $ftp($i) attach-agent $tcp($i)
}

# flow monitor
set link_node [$ns link $n(10) $n(11)]
set flow_monitor [$ns makeflowmon Fid]
$ns attach-fmon $link_node $flow_monitor
set fcl [$flow_monitor classifier]


# schedule events for all the flows
$ns at 1 "record_stat"


for {set i 0} {$i < 10} {incr i} {
    
    $ns at 0.1 "$ftp($i) start"
}

for {set i 0} {$i < 10} {incr i} {

    $ns at 6 "$ftp($i) stop"
}


# call the finish procedure after 6 seconds of simulation time
$ns at 6 "finish"

# run the simulation
$ns run


