# create a simulator object
set ns [new Simulator]

# open the nam trace file
set nam_trace_fd [open Reno_Fast1.nam w]
$ns namtrace-all $nam_trace_fd
set trace_fd [open Reno_Fast1.tr w]

# open the measurement output files
set throughput_trace_fd(0) [open Reno_Fast1_throughput1.tr w]
set throughput_trace_fd(1) [open Reno_Fast1_throughput2.tr w]
set total_throughput(1) 0
set total_throughput(0) 0


set packetSize 1000

# define different colors for nam data flows
$ns color 0 Green
$ns color 1 Blue
$ns color 2 Red
$ns color 3 Yellow

# define a 'finish' procedure
proc finish {} {
        global ns nam_trace_fd trace_fd
        global trace_fd
   	global throughput_trace_fd
        global total_throughput

        puts $throughput_trace_fd(0) "$total_throughput(0)"
        puts $throughput_trace_fd(1) "$total_throughput(1)"

	# close the nam trace file
        $ns flush-trace
        close $nam_trace_fd
   
	 # close the measurement files
	 close $trace_fd
	 close $throughput_trace_fd(0)
	 close $throughput_trace_fd(1)
   
	# execute nam on the trace file
        # exec nam tcp_c.nam &
        exit 0
}

proc record_stat {} {
        global flow_monitor tcp packetSize flowstats
        global trace_fd throughput_trace_fd
        global total_throughput        

	# get an instance of the simulator
	set ns [Simulator instance]

	# set the time after which the procedure should be called again
        set time 0.01

	# get the current time
        set now [$ns now]
	for {set i 0} {$i < 2} {incr i} {             
		# how many bytes have been received by the traffic sinks?
        	set flow_classifier [$flow_monitor classifier]
	
		set flow_fd [$flow_classifier lookup auto 0 0 $i]
		if { $flow_fd != "" } {
		
			# calculate the throughput and write it to the files
		    set temp [expr [$flow_fd set bdepartures_]*8/$time]
			puts $throughput_trace_fd($i) "$now \t $temp"
			set $total_throughput($i) [expr $total_throughput($i) + $temp]
			$flow_fd set barrivals_ 0
			$flow_fd set bdepartures_ 0
			$flow_fd set bdrops_ 0		
	
		    }
	
     	}
    # re-schedule the procedure
    $ns at [expr $now+$time] "record_stat"
}


# create 6 nodes
for {set i 0} {$i < 6} {incr i} {
  set n($i) [$ns node]
}


# create links between the nodes
$ns duplex-link $n(0) $n(2) 80Mb 10ms DropTail
$ns duplex-link $n(3) $n(4) 80Mb 10ms DropTail
$ns duplex-link $n(1) $n(2) 80Mb 10ms DropTail
$ns duplex-link $n(3) $n(5) 80Mb 10ms DropTail

$ns duplex-link $n(2) $n(3) 30Mb 10ms DropTail 
$ns queue-limit $n(2) $n(3) 200

# monitor the queue for the link between node 2 and node 3
$ns duplex-link-op $n(2) $n(3) queuePos 0.5

#Setting for nam
$ns duplex-link-op $n(0) $n(2) orient right-down
$ns duplex-link-op $n(1) $n(2) orient right-up
$ns duplex-link-op $n(2) $n(3) orient right
$ns duplex-link-op $n(3) $n(4) orient right-down
$ns duplex-link-op $n(3) $n(5) orient right-up


# TCP traffic source
# create a TCP agent and attach it to nodes

set tcp(0) [new Agent/TCP/Reno]
$ns attach-agent $n(0) $tcp(0)
$tcp(0) set fid_ 1
$tcp(0) set class_ 1
# set packet length, please don't remove below line!!!
$tcp(0) set window_ 9999





set tcp(1) [new Agent/TCP/Fast]
$ns attach-agent $n(1) $tcp(1)
$tcp(1) set fid_ 2
$tcp(1) set class_ 2
# set packet length, please don't remove below line!!!
$tcp(1) set window_ 9999
$tcp(1) set alpha_ 200
$tcp(1) set beta_ 200

set sink(0) [new Agent/TCPSink]
$ns attach-agent $n(4) $sink(0)
# connect both agents
$ns connect $tcp(0) $sink(0)

# create an FTP source "application";
set ftp(0) [new Application/FTP]
$ftp(0) attach-agent $tcp(0)

set sink(1) [new Agent/TCPSink/Sack1]
$ns attach-agent $n(5) $sink(1)
# connect both agents
$ns connect $tcp(1) $sink(1)

# create an FTP source "application";
set ftp(1) [new Application/FTP]
$ftp(1) attach-agent $tcp(1)


# flow monitor
set link_node [$ns link $n(2) $n(3)]
set flow_monitor [$ns makeflowmon Fid]
$ns attach-fmon $link_node $flow_monitor
set fcl [$flow_monitor classifier]


# schedule events for all the flows
$ns at 0.1 "record_stat"

    $ns at 0.1 "$ftp(0) start"
    $ns at 0.1 "$ftp(1) start"

    $ns at 8.0 "$ftp(0) stop"
    $ns at 8.0 "$ftp(1) stop"


# call the finish procedure after 6 seconds of simulation time
$ns at 8 "finish"

# run the simulation
$ns run




