# create a simulator object
set ns [new Simulator]

# open the nam trace file
set nam_trace_fd [open tcp_A.nam w]
$ns namtrace-all $nam_trace_fd
set trace_fd [open tcp_A.tr w]

# open the measurement output files
set throughput_trace_fd [open tcp_A_throughput.tr w]
set drop_trace_fd [open tcp_A_drop.tr w]
set cwind_trace_fd [open tcp_A_cwind.tr w]
set seq_trace_fd [open tcp_A_seq.tr w]
set ack_trace_fd [open tcp_A_ack.tr w]

set packetSize 1000

# define different colors for nam data flows
$ns color 0 Green
$ns color 1 Blue
$ns color 2 Red
$ns color 3 Yellow

# define a 'finish' procedure
proc finish {} {
        global ns nam_trace_fd trace_fd
        global trace_fd throughput_trace_fd drop_trace_fd
        global cwind_trace_fd seq_trace_fd ack_trace_fd

	# close the nam trace file
        $ns flush-trace
        close $nam_trace_fd

	# close the measurement files
	close $trace_fd
	close $throughput_trace_fd
	close $drop_trace_fd
	close $cwind_trace_fd
	close $seq_trace_fd
	close $ack_trace_fd

	# execute nam on the trace file
        # exec nam tcp_tahoe.nam &
        exit 0
}

proc record_stat {} {
        global flow_monitor tcp packetSize
        global trace_fd throughput_trace_fd drop_trace_fd
        global cwind_trace_fd seq_trace_fd ack_trace_fd

	# get an instance of the simulator
	set ns [Simulator instance]

	# set the time after which the procedure should be called again
        set time 0.01

	# get the current time
        set now [$ns now]

	# how many bytes have been received by the traffic sinks?
        set packet_arrival [$flow_monitor set parrivals_]
        set packet_departure [$flow_monitor set pdepartures_]
        set packet_drop [$flow_monitor set pdrops_]
        set flow_classifier [$flow_monitor classifier]
	
	set flow_fd [$flow_classifier lookup auto 0 0 1]
	if { $flow_fd != "" } {
		# calculate the throughput and write it to the files
		puts $throughput_trace_fd "$now \t [expr [$flow_monitor set bdepartures_]*8/$time]"

		# calculate the dropped packets and write it to the files
		puts $drop_trace_fd "$now \t [expr [$flow_monitor set bdrops_]/$packetSize]"

		# record current congestion window
		puts $cwind_trace_fd "$now \t [$tcp set cwnd_]"

		# record current sequence number
		puts $seq_trace_fd "$now \t [$tcp set maxseq_]"

		# record acknowledge number
		puts $ack_trace_fd "$now \t [$tcp set ack_]"
	}

     	# reset the bytes_ values on the traffic sinks
        $flow_monitor set barrivals_ 0
        $flow_monitor set bdepartures_ 0
        $flow_monitor set bdrops_ 0

     	# re-schedule the procedure
        $ns at [expr $now+$time] "record_stat"
}


# create four nodes
set node1 [$ns node]
set node2 [$ns node]
set node3 [$ns node]
set node4 [$ns node]

# create links between the nodes
$ns duplex-link $node1 $node3 2Mb 10ms DropTail
$ns duplex-link $node2 $node3 2Mb 10ms DropTail
$ns duplex-link $node3 $node4 1Mb 10ms DropTail 
$ns queue-limit $node3 $node4 4

# flow monitor
set link_node3_node4 [$ns link $node3 $node4]
set flow_monitor [$ns makeflowmon Fid]
$ns attach-fmon $link_node3_node4 $flow_monitor

# monitor the queue for the link between node 2 and node 3
$ns duplex-link-op $node3 $node4 queuePos 0.5

#Setting for nam
$ns duplex-link-op $node1 $node3 orient right-down
$ns duplex-link-op $node2 $node3 orient right-up
$ns duplex-link-op $node3 $node4 orient right

# TCP traffic source
# create a TCP agent and attach it to node node1
set tcp [new Agent/TCP/TCP_A]
$ns attach-agent $node1 $tcp
$tcp set fid_ 1
$tcp set class_ 1

# set packet length, please don't remove below line!!!
$tcp set window_ 9999
# window_ * (packetsize_ + 40) / RTT
$tcp set window_ 30

# create a TCP sink agent and attach it to node node4
set sink [new Agent/TCPSink/Sink_A]
$ns attach-agent $node4 $sink

# connect both agents
$ns connect $tcp $sink

# create an FTP source "application";
set ftp [new Application/FTP]
$ftp attach-agent $tcp

# UDP traffic source
# create a UDP agent and attach it to node 2
set udp [new Agent/UDP]
$ns attach-agent $node2 $udp

# create a CBR traffic source and attach it to udp
set cbr [new Application/Traffic/CBR]
$cbr set packetSize $packetSize
$cbr set rate 0.5mb
$cbr attach-agent $udp

# creat a Null agent (a traffic sink) and attach it to node 4
set null [new Agent/Null]
$ns attach-agent $node4 $null
$ns connect $udp $null

# schedule events for all the flows
$ns at 0.1 "record_stat"
$ns at 0.1 "$ftp start"
$ns at 4.0 "$cbr start"
$ns at 5.0 "$cbr stop"
$ns at 8.0 "$ftp stop"

# call the finish procedure after 6 seconds of simulation time
$ns at 8 "finish"

# run the simulation
$ns run

